class IncorrectApiUsageError(Exception): ...
class UrlError(Exception): ...
class ConnectionClosedError(Exception): ...
class TransactionClosedError(Exception): ...
class BuilderConsumedError(Exception): ...
class DecodeError(Exception): ...
